/*
 * Generated by asn1c-0.9.28 (http://lionet.info/asn1c)
 * From ASN.1 module "HNBAP-IEs"
 * 	found in "../../asn1/hnbap/HNBAP-IEs.asn"
 * 	`asn1c -R -fcompound-names`
 */

#include <osmocom/hnbap/HNBAP_TransportInfo.h>

static asn_per_constraints_t asn_PER_type_transportAssociation_constr_3 GCC_NOTUSED = {
	{ APC_CONSTRAINED | APC_EXTENSIBLE,  1,  1,  0l,  1l }	/* (0..1,...) */,
	{ APC_UNCONSTRAINED,	-1, -1,  0,  0 },
	0, 0	/* No PER value map */
};
static asn_TYPE_member_t asn_MBR_transportAssociation_3[] = {
	{ ATF_NOFLAGS, 0, offsetof(struct HNBAP_TransportInfo__transportAssociation, choice.gtp_TEI),
		(ASN_TAG_CLASS_CONTEXT | (0 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_HNBAP_GTP_TEI,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"gtp-TEI"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct HNBAP_TransportInfo__transportAssociation, choice.bindingID),
		(ASN_TAG_CLASS_CONTEXT | (1 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_HNBAP_BindingID,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"bindingID"
		},
};
static const asn_TYPE_tag2member_t asn_MAP_transportAssociation_tag2el_3[] = {
    { (ASN_TAG_CLASS_CONTEXT | (0 << 2)), 0, 0, 0 }, /* gtp-TEI */
    { (ASN_TAG_CLASS_CONTEXT | (1 << 2)), 1, 0, 0 } /* bindingID */
};
static asn_CHOICE_specifics_t asn_SPC_transportAssociation_specs_3 = {
	sizeof(struct HNBAP_TransportInfo__transportAssociation),
	offsetof(struct HNBAP_TransportInfo__transportAssociation, _asn_ctx),
	offsetof(struct HNBAP_TransportInfo__transportAssociation, present),
	sizeof(((struct HNBAP_TransportInfo__transportAssociation *)0)->present),
	asn_MAP_transportAssociation_tag2el_3,
	2,	/* Count of tags in the map */
	0,
	2	/* Extensions start */
};
static /* Use -fall-defs-global to expose */
asn_TYPE_descriptor_t asn_DEF_transportAssociation_3 = {
	"transportAssociation",
	"transportAssociation",
	CHOICE_free,
	CHOICE_print,
	CHOICE_constraint,
	CHOICE_decode_ber,
	CHOICE_encode_der,
	CHOICE_decode_xer,
	CHOICE_encode_xer,
	CHOICE_decode_uper,
	CHOICE_encode_uper,
	CHOICE_decode_aper,
	CHOICE_encode_aper,
	CHOICE_outmost_tag,
	0,	/* No effective tags (pointer) */
	0,	/* No effective tags (count) */
	0,	/* No tags (pointer) */
	0,	/* No tags (count) */
	&asn_PER_type_transportAssociation_constr_3,
	asn_MBR_transportAssociation_3,
	2,	/* Elements count */
	&asn_SPC_transportAssociation_specs_3	/* Additional specs */
};

static asn_TYPE_member_t asn_MBR_HNBAP_TransportInfo_1[] = {
	{ ATF_NOFLAGS, 0, offsetof(struct HNBAP_TransportInfo, transportLayerAddress),
		(ASN_TAG_CLASS_CONTEXT | (0 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_HNBAP_TransportLayerAddress,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"transportLayerAddress"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct HNBAP_TransportInfo, transportAssociation),
		(ASN_TAG_CLASS_CONTEXT | (1 << 2)),
		+1,	/* EXPLICIT tag at current level */
		&asn_DEF_transportAssociation_3,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"transportAssociation"
		},
	{ ATF_POINTER, 1, offsetof(struct HNBAP_TransportInfo, iE_Extensions),
		(ASN_TAG_CLASS_CONTEXT | (2 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_HNBAP_IE_Extensions,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"iE-Extensions"
		},
};
static const int asn_MAP_HNBAP_TransportInfo_oms_1[] = { 2 };
static const ber_tlv_tag_t asn_DEF_HNBAP_TransportInfo_tags_1[] = {
	(ASN_TAG_CLASS_UNIVERSAL | (16 << 2))
};
static const asn_TYPE_tag2member_t asn_MAP_HNBAP_TransportInfo_tag2el_1[] = {
    { (ASN_TAG_CLASS_CONTEXT | (0 << 2)), 0, 0, 0 }, /* transportLayerAddress */
    { (ASN_TAG_CLASS_CONTEXT | (1 << 2)), 1, 0, 0 }, /* transportAssociation */
    { (ASN_TAG_CLASS_CONTEXT | (2 << 2)), 2, 0, 0 } /* iE-Extensions */
};
static asn_SEQUENCE_specifics_t asn_SPC_HNBAP_TransportInfo_specs_1 = {
	sizeof(struct HNBAP_TransportInfo),
	offsetof(struct HNBAP_TransportInfo, _asn_ctx),
	asn_MAP_HNBAP_TransportInfo_tag2el_1,
	3,	/* Count of tags in the map */
	asn_MAP_HNBAP_TransportInfo_oms_1,	/* Optional members */
	1, 0,	/* Root/Additions */
	2,	/* Start extensions */
	4	/* Stop extensions */
};
asn_TYPE_descriptor_t asn_DEF_HNBAP_TransportInfo = {
	"HNBAP_TransportInfo",
	"HNBAP_TransportInfo",
	SEQUENCE_free,
	SEQUENCE_print,
	SEQUENCE_constraint,
	SEQUENCE_decode_ber,
	SEQUENCE_encode_der,
	SEQUENCE_decode_xer,
	SEQUENCE_encode_xer,
	SEQUENCE_decode_uper,
	SEQUENCE_encode_uper,
	SEQUENCE_decode_aper,
	SEQUENCE_encode_aper,
	0,	/* Use generic outmost tag fetcher */
	asn_DEF_HNBAP_TransportInfo_tags_1,
	sizeof(asn_DEF_HNBAP_TransportInfo_tags_1)
		/sizeof(asn_DEF_HNBAP_TransportInfo_tags_1[0]), /* 1 */
	asn_DEF_HNBAP_TransportInfo_tags_1,	/* Same as above */
	sizeof(asn_DEF_HNBAP_TransportInfo_tags_1)
		/sizeof(asn_DEF_HNBAP_TransportInfo_tags_1[0]), /* 1 */
	0,	/* No PER visible constraints */
	asn_MBR_HNBAP_TransportInfo_1,
	3,	/* Elements count */
	&asn_SPC_HNBAP_TransportInfo_specs_1	/* Additional specs */
};

