/*
 * Generated by asn1c-0.9.28 (http://lionet.info/asn1c)
 * From ASN.1 module "HNBAP-IEs"
 * 	found in "../../asn1/hnbap/HNBAP-IEs.asn"
 * 	`asn1c -R -fcompound-names`
 */

#ifndef	_HNBAP_Access_stratum_release_indicator_H_
#define	_HNBAP_Access_stratum_release_indicator_H_


#include <asn_application.h>

/* Including external dependencies */
#include <NativeEnumerated.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dependencies */
typedef enum HNBAP_Access_stratum_release_indicator {
	HNBAP_Access_stratum_release_indicator_r99	= 0,
	HNBAP_Access_stratum_release_indicator_rel_4	= 1,
	HNBAP_Access_stratum_release_indicator_rel_5	= 2,
	HNBAP_Access_stratum_release_indicator_rel_6	= 3,
	HNBAP_Access_stratum_release_indicator_rel_7	= 4,
	HNBAP_Access_stratum_release_indicator_rel_8_and_beyond	= 5
	/*
	 * Enumeration is extensible
	 */
} e_HNBAP_Access_stratum_release_indicator;

/* HNBAP_Access-stratum-release-indicator */
typedef long	 HNBAP_Access_stratum_release_indicator_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_HNBAP_Access_stratum_release_indicator;
asn_struct_free_f HNBAP_Access_stratum_release_indicator_free;
asn_struct_print_f HNBAP_Access_stratum_release_indicator_print;
asn_constr_check_f HNBAP_Access_stratum_release_indicator_constraint;
ber_type_decoder_f HNBAP_Access_stratum_release_indicator_decode_ber;
der_type_encoder_f HNBAP_Access_stratum_release_indicator_encode_der;
xer_type_decoder_f HNBAP_Access_stratum_release_indicator_decode_xer;
xer_type_encoder_f HNBAP_Access_stratum_release_indicator_encode_xer;
per_type_decoder_f HNBAP_Access_stratum_release_indicator_decode_uper;
per_type_encoder_f HNBAP_Access_stratum_release_indicator_encode_uper;
per_type_decoder_f HNBAP_Access_stratum_release_indicator_decode_aper;
per_type_encoder_f HNBAP_Access_stratum_release_indicator_encode_aper;

#ifdef __cplusplus
}
#endif

#endif	/* _HNBAP_Access_stratum_release_indicator_H_ */
#include <asn_internal.h>
