/*
 * Generated by asn1c-0.9.28 (http://lionet.info/asn1c)
 * From ASN.1 module "RANAP-IEs"
 * 	found in "../../asn1/ranap/RANAP-IEs.asn"
 * 	`asn1c -fcompound-names -R`
 */

#ifndef	_RANAP_RNCTraceInformation_H_
#define	_RANAP_RNCTraceInformation_H_


#include <asn_application.h>

/* Including external dependencies */
#include <osmocom/ranap/RANAP_TraceReference.h>
#include <NativeEnumerated.h>
#include <osmocom/ranap/RANAP_EquipmentsToBeTraced.h>
#include <osmocom/ranap/RANAP_IE-Extensions.h>
#include <constr_SEQUENCE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dependencies */
typedef enum RANAP_RNCTraceInformation__traceActivationIndicato {
	RANAP_RNCTraceInformation__traceActivationIndicato_activated	= 0,
	RANAP_RNCTraceInformation__traceActivationIndicato_deactivated	= 1
} e_RANAP_RNCTraceInformation__traceActivationIndicato;

/* RANAP_RNCTraceInformation */
typedef struct RANAP_RNCTraceInformation {
	RANAP_TraceReference_t	 traceReference;
	long	 traceActivationIndicato;
	RANAP_EquipmentsToBeTraced_t	*equipmentsToBeTraced	/* OPTIONAL */;
	RANAP_IE_Extensions_t	*iE_Extensions	/* OPTIONAL */;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} RANAP_RNCTraceInformation_t;

/* Implementation */
/* extern asn_TYPE_descriptor_t asn_DEF_traceActivationIndicato_3;	// (Use -fall-defs-global to expose) */
extern asn_TYPE_descriptor_t asn_DEF_RANAP_RNCTraceInformation;

#ifdef __cplusplus
}
#endif

#endif	/* _RANAP_RNCTraceInformation_H_ */
#include <asn_internal.h>
