/*
 * Generated by asn1c-0.9.28 (http://lionet.info/asn1c)
 * From ASN.1 module "HNBAP-IEs"
 * 	found in "../../asn1/hnbap/HNBAP-IEs.asn"
 * 	`asn1c -R -fcompound-names`
 */

#ifndef	_HNBAP_CGI_H_
#define	_HNBAP_CGI_H_


#include <asn_application.h>

/* Including external dependencies */
#include <osmocom/hnbap/HNBAP_PLMNidentity.h>
#include <osmocom/hnbap/HNBAP_LAC.h>
#include <osmocom/hnbap/HNBAP_CI.h>
#include <osmocom/hnbap/HNBAP_IE-Extensions.h>
#include <constr_SEQUENCE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* HNBAP_CGI */
typedef struct HNBAP_CGI {
	HNBAP_PLMNidentity_t	 pLMNidentity;
	HNBAP_LAC_t	 lAC;
	HNBAP_CI_t	 cI;
	HNBAP_IE_Extensions_t	*iE_Extensions	/* OPTIONAL */;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} HNBAP_CGI_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_HNBAP_CGI;

#ifdef __cplusplus
}
#endif

#endif	/* _HNBAP_CGI_H_ */
#include <asn_internal.h>
