/*
 * Generated by asn1c-0.9.28 (http://lionet.info/asn1c)
 * From ASN.1 module "HNBAP-IEs"
 * 	found in "../../asn1/hnbap/HNBAP-IEs.asn"
 * 	`asn1c -R -fcompound-names`
 */

#ifndef	_HNBAP_HNB_RNL_Identity_H_
#define	_HNBAP_HNB_RNL_Identity_H_


#include <asn_application.h>

/* Including external dependencies */
#include <osmocom/hnbap/HNBAP_HNB-Cell-Identifier.h>
#include <constr_CHOICE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dependencies */
typedef enum HNBAP_HNB_RNL_Identity_PR {
	HNBAP_HNB_RNL_Identity_PR_NOTHING,	/* No components present */
	HNBAP_HNB_RNL_Identity_PR_hNB_Identity_as_Cell_Identifier,
	/* Extensions may appear below */
	
} HNBAP_HNB_RNL_Identity_PR;

/* HNBAP_HNB-RNL-Identity */
typedef struct HNBAP_HNB_RNL_Identity {
	HNBAP_HNB_RNL_Identity_PR present;
	union HNBAP_HNB_RNL_Identity_u {
		HNBAP_HNB_Cell_Identifier_t	 hNB_Identity_as_Cell_Identifier;
		/*
		 * This type is extensible,
		 * possible extensions are below.
		 */
	} choice;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} HNBAP_HNB_RNL_Identity_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_HNBAP_HNB_RNL_Identity;

#ifdef __cplusplus
}
#endif

#endif	/* _HNBAP_HNB_RNL_Identity_H_ */
#include <asn_internal.h>
